@@ -29,6 +29,7 @@ namespace arrow {
 namespace r {
 struct symbols {
   static SEXP units;
+  static SEXP tzone;
   static SEXP xp;
   static SEXP dot_Internal;
   static SEXP inspect;
@@ -196,6 +197,7 @@ inline std::shared_ptr<T> extract(SEXP x) {
 
 #if defined(ARROW_R_WITH_ARROW)
 #include <arrow/api.h>
+#include <arrow/c/bridge.h>
 #include <arrow/compute/api.h>
 #include <arrow/csv/reader.h>
 #include <arrow/dataset/api.h>
@@ -210,8 +212,10 @@ inline std::shared_ptr<T> extract(SEXP x) {
 #include <arrow/json/reader.h>
 #include <arrow/result.h>
 #include <arrow/type.h>
+#include <arrow/type_fwd.h>
 #include <arrow/util/checked_cast.h>
 #include <arrow/util/compression.h>
+#include <arrow/util/iterator.h>
 #include <arrow/util/ubsan.h>
 #include <arrow/visitor_inline.h>
 #include <parquet/arrow/reader.h>
@@ -289,6 +293,12 @@ class RBuffer : public MutableBuffer {
 
 std::shared_ptr<arrow::DataType> InferArrowTypeFromFactor(SEXP);
 
+void validate_slice_offset(int offset, int len);
+
+void validate_slice_length(int length, int available);
+
+void validate_index(int i, int len);
+
 }  // namespace r
 }  // namespace arrow
 
