@@ -57,31 +57,25 @@ bool ipc___Message__Equals(const std::unique_ptr<arrow::ipc::Message>& x,
 std::shared_ptr<arrow::RecordBatch> ipc___ReadRecordBatch__Message__Schema(
     const std::unique_ptr<arrow::ipc::Message>& message,
     const std::shared_ptr<arrow::Schema>& schema) {
-  std::shared_ptr<arrow::RecordBatch> batch;
-
   // TODO: perhaps this should come from the R side
   arrow::ipc::DictionaryMemo memo;
-  STOP_IF_NOT_OK(arrow::ipc::ReadRecordBatch(*message, schema, &memo, &batch));
-  return batch;
+  return VALUE_OR_STOP(arrow::ipc::ReadRecordBatch(
+      *message, schema, &memo, arrow::ipc::IpcReadOptions::Defaults()));
 }
 
 // [[arrow::export]]
 std::shared_ptr<arrow::Schema> ipc___ReadSchema_InputStream(
     const std::shared_ptr<arrow::io::InputStream>& stream) {
-  std::shared_ptr<arrow::Schema> schema;
   // TODO: promote to function argument
   arrow::ipc::DictionaryMemo memo;
-  STOP_IF_NOT_OK(arrow::ipc::ReadSchema(stream.get(), &memo, &schema));
-  return schema;
+  return VALUE_OR_STOP(arrow::ipc::ReadSchema(stream.get(), &memo));
 }
 
 // [[arrow::export]]
 std::shared_ptr<arrow::Schema> ipc___ReadSchema_Message(
     const std::unique_ptr<arrow::ipc::Message>& message) {
-  std::shared_ptr<arrow::Schema> schema;
   arrow::ipc::DictionaryMemo empty_memo;
-  STOP_IF_NOT_OK(arrow::ipc::ReadSchema(*message, &empty_memo, &schema));
-  return schema;
+  return VALUE_OR_STOP(arrow::ipc::ReadSchema(*message, &empty_memo));
 }
 
 //--------- MessageReader
@@ -95,17 +89,13 @@ std::unique_ptr<arrow::ipc::MessageReader> ipc___MessageReader__Open(
 // [[arrow::export]]
 std::unique_ptr<arrow::ipc::Message> ipc___MessageReader__ReadNextMessage(
     const std::unique_ptr<arrow::ipc::MessageReader>& reader) {
-  std::unique_ptr<arrow::ipc::Message> message;
-  STOP_IF_NOT_OK(reader->ReadNextMessage(&message));
-  return message;
+  return VALUE_OR_STOP(reader->ReadNextMessage());
 }
 
 // [[arrow::export]]
 std::unique_ptr<arrow::ipc::Message> ipc___ReadMessage(
     const std::shared_ptr<arrow::io::InputStream>& stream) {
-  std::unique_ptr<arrow::ipc::Message> message;
-  STOP_IF_NOT_OK(arrow::ipc::ReadMessage(stream.get(), &message));
-  return message;
+  return VALUE_OR_STOP(arrow::ipc::ReadMessage(stream.get()));
 }
 
 #endif
