@@ -26,41 +26,30 @@ test_that("read_table handles various input streams (ARROW-3450, ARROW-3505)", {
   tab <- Table$create(!!!tbl)
 
   tf <- tempfile()
+  on.exit(unlink(tf))
   write_arrow(tab, tf)
 
   bytes <- write_arrow(tab, raw())
 
-  tab1 <- read_table(tf)
-  tab2 <- read_table(normalizePath(tf))
+  tab1 <- read_arrow(tf, as_data_frame = FALSE)
+  tab2 <- read_feather(normalizePath(tf), as_data_frame = FALSE)
 
   readable_file <- ReadableFile$create(tf)
-  file_reader1 <- RecordBatchFileReader$create(readable_file)
-  tab3 <- read_table(file_reader1)
+  tab3 <- read_arrow(readable_file, as_data_frame = FALSE)
   readable_file$close()
 
   mmap_file <- mmap_open(tf)
-  file_reader2 <- RecordBatchFileReader$create(mmap_file)
-  tab4 <- read_table(file_reader2)
+  # check for deprecation message
+  expect_deprecated(
+    tab4 <- read_table(mmap_file),
+    "read_arrow"
+  )
   mmap_file$close()
 
-  tab5 <- read_table(bytes)
-
-  stream_reader <- RecordBatchStreamReader$create(bytes)
-  tab6 <- read_table(stream_reader)
-
-  file_reader <- RecordBatchFileReader$create(tf)
-  tab7 <- read_table(file_reader)
-
   expect_equal(tab, tab1)
   expect_equal(tab, tab2)
   expect_equal(tab, tab3)
   expect_equal(tab, tab4)
-  expect_equal(tab, tab5)
-  expect_equal(tab, tab6)
-  expect_equal(tab, tab7)
-
-  unlink(tf)
-
 })
 
 test_that("Table cast (ARROW-3741)", {
@@ -82,6 +71,25 @@ test_that("Table dim() and nrow() (ARROW-3816)", {
   expect_equal(nrow(tab), 10L)
 })
 
+test_that("Table $column and $field", {
+  tab <- Table$create(x = 1:10, y  = 1:10)
+
+  expect_equal(tab$field(0), field("x", int32()))
+
+  # input validation
+  expect_error(tab$column(NA), "'i' cannot be NA")
+  expect_error(tab$column(-1), "subscript out of bounds")
+  expect_error(tab$column(1000), "subscript out of bounds")
+  expect_error(tab$column(1:2), class = "Rcpp::not_compatible")
+  expect_error(tab$column("one"), class = "Rcpp::not_compatible")
+
+  expect_error(tab$field(NA), "'i' cannot be NA")
+  expect_error(tab$field(-1), "subscript out of bounds")
+  expect_error(tab$field(1000), "subscript out of bounds")
+  expect_error(tab$field(1:2), class = "Rcpp::not_compatible")
+  expect_error(tab$field("one"), class = "Rcpp::not_compatible")
+})
+
 test_that("[, [[, $ for Table", {
   tbl <- tibble::tibble(
     int = 1:10,
@@ -120,12 +128,35 @@ test_that("[, [[, $ for Table", {
   expect_vector(tab[[4]], tbl$chr)
   expect_null(tab$qwerty)
   expect_null(tab[["asdf"]])
-  expect_error(tab[[c(4, 3)]], 'length(i) not equal to 1', fixed = TRUE)
+  expect_error(tab[[c(4, 3)]], class = "Rcpp::not_compatible")
   expect_error(tab[[NA]], "'i' must be character or numeric, not logical")
   expect_error(tab[[NULL]], "'i' must be character or numeric, not NULL")
   expect_error(tab[[c("asdf", "jkl;")]], 'length(name) not equal to 1', fixed = TRUE)
 })
 
+test_that("Table$Slice", {
+  tab2 <- tab$Slice(5)
+  expect_data_frame(tab2, tbl[6:10,])
+
+  tab3 <- tab$Slice(5, 2)
+  expect_data_frame(tab3, tbl[6:7,])
+
+  # Input validation
+  expect_error(tab$Slice("ten"), class = "Rcpp::not_compatible")
+  expect_error(tab$Slice(NA_integer_), "Slice 'offset' cannot be NA")
+  expect_error(tab$Slice(NA), "Slice 'offset' cannot be NA")
+  expect_error(tab$Slice(10, "ten"), class = "Rcpp::not_compatible")
+  expect_error(tab$Slice(10, NA_integer_), "Slice 'length' cannot be NA")
+  expect_error(tab$Slice(NA_integer_, NA_integer_), "Slice 'offset' cannot be NA")
+  expect_error(tab$Slice(c(10, 10)), class = "Rcpp::not_compatible")
+  expect_error(tab$Slice(10, c(10, 10)), class = "Rcpp::not_compatible")
+  expect_error(tab$Slice(1000), "Slice 'offset' greater than array length")
+  expect_error(tab$Slice(-1), "Slice 'offset' cannot be negative")
+  expect_error(tab3$Slice(10, 10), "Slice 'offset' greater than array length")
+  expect_error(tab$Slice(10, -1), "Slice 'length' cannot be negative")
+  expect_error(tab$Slice(-1, 10), "Slice 'offset' cannot be negative")
+})
+
 test_that("head and tail on Table", {
   tbl <- tibble::tibble(
     int = 1:10,
@@ -136,12 +167,16 @@ test_that("head and tail on Table", {
   )
   tab <- Table$create(tbl)
 
-  expect_identical(as.data.frame(head(tab)), head(tbl))
-  expect_identical(as.data.frame(head(tab, 4)), head(tbl, 4))
-  expect_identical(as.data.frame(head(tab, -4)), head(tbl, -4))
-  expect_identical(as.data.frame(tail(tab)), tail(tbl))
-  expect_identical(as.data.frame(tail(tab, 4)), tail(tbl, 4))
-  expect_identical(as.data.frame(tail(tab, -4)), tail(tbl, -4))
+  expect_data_frame(head(tab), head(tbl))
+  expect_data_frame(head(tab, 4), head(tbl, 4))
+  expect_data_frame(head(tab, 40), head(tbl, 40))
+  expect_data_frame(head(tab, -4), head(tbl, -4))
+  expect_data_frame(head(tab, -40), head(tbl, -40))
+  expect_data_frame(tail(tab), tail(tbl))
+  expect_data_frame(tail(tab, 4), tail(tbl, 4))
+  expect_data_frame(tail(tab, 40), tail(tbl, 40))
+  expect_data_frame(tail(tab, -4), tail(tbl, -4))
+  expect_data_frame(tail(tab, -40), tail(tbl, -40))
 })
 
 test_that("Table print method", {
@@ -250,9 +285,28 @@ test_that("==.Table", {
   expect_equal(tab1, tab2)
 })
 
+test_that("Table$Equals(check_metadata)", {
+  tab1 <- Table$create(x = 1:2, y = c("a", "b"))
+  tab2 <- Table$create(x = 1:2, y = c("a", "b"),
+                       schema = tab1$schema$WithMetadata(list(some="metadata")))
+
+  expect_is(tab1, "Table")
+  expect_is(tab2, "Table")
+  expect_false(tab1$schema$HasMetadata)
+  expect_true(tab2$schema$HasMetadata)
+  expect_match(tab2$schema$metadata, "some: metadata", fixed = TRUE)
+
+  expect_true(tab1 == tab2)
+  expect_true(tab1$Equals(tab2))
+  expect_false(tab1$Equals(tab2, check_metadata = TRUE))
+
+  expect_failure(expect_equal(tab1, tab2))  # expect_equal has check_metadata=TRUE
+  expect_equivalent(tab1, tab2)  # expect_equivalent has check_metadata=FALSE
+
+  expect_false(tab1$Equals(24)) # Not a Table
+})
+
 test_that("Table handles null type (ARROW-7064)", {
   tab <- Table$create(a = 1:10, n = vctrs::unspecified(10))
   expect_equal(tab$schema,  schema(a = int32(), n = null()))
 })
-
-
