@@ -295,14 +295,14 @@ test_that("time type unit validation", {
   expect_equal(time32(), time32(TimeUnit$MILLI))
   expect_error(time32(4), '"unit" should be one of 1 or 0')
   expect_error(time32(NULL), '"unit" should be one of "ms" or "s"')
-  expect_error(time32("years"), "'arg' should be one of")
+  expect_match_arg_error(time32("years"))
 
   expect_equal(time64(TimeUnit$NANO), time64("n"))
   expect_equal(time64(TimeUnit$MICRO), time64("us"))
   expect_equal(time64(), time64(TimeUnit$NANO))
   expect_error(time64(4), '"unit" should be one of 3 or 2')
   expect_error(time64(NULL), '"unit" should be one of "ns" or "us"')
-  expect_error(time64("years"), "'arg' should be one of")
+  expect_match_arg_error(time64("years"))
 })
 
 test_that("timestamp type input validation", {
@@ -314,12 +314,11 @@ test_that("timestamp type input validation", {
   )
   expect_error(
     timestamp(timezone = 1231231),
-    "timezone is not a character vector"
+    "timezone is not a string"
   )
   expect_error(
     timestamp(timezone = c("not", "a", "timezone")),
-    "length(timezone) not equal to 1",
-    fixed = TRUE
+    "timezone is not a string"
   )
 })
 
@@ -379,4 +378,36 @@ test_that("DictionaryType validation", {
     dictionary(utf8(), int32()),
     "Dictionary index type should be signed integer, got string"
   )
+  expect_error(dictionary(4, utf8()), 'index_type must be a "DataType"')
+  expect_error(dictionary(int8(), "strings"), 'value_type must be a "DataType"')
+})
+
+test_that("decimal type and validation", {
+  expect_error(decimal())
+  expect_error(decimal("four"), '"precision" must be an integer')
+  expect_error(decimal(4))
+  expect_error(decimal(4, "two"), '"scale" must be an integer')
+  expect_error(decimal(NA, 2), '"precision" must be an integer')
+  expect_error(decimal(0, 2), "Invalid: Decimal precision out of range: 0")
+  expect_error(decimal(100, 2), "Invalid: Decimal precision out of range: 100")
+  expect_error(decimal(4, NA), '"scale" must be an integer')
+
+  expect_is(decimal(4, 2), "Decimal128Type")
+
+})
+
+test_that("Binary", {
+  expect_is(binary(), "Binary")
+  expect_equal(binary()$ToString(), "binary")
+})
+
+test_that("FixedSizeBinary", {
+  expect_is(binary(4), "FixedSizeBinary")
+  expect_equal(binary(4)$ToString(), "fixed_size_binary[4]")
+
+  # input validation
+  expect_error(binary(NA), "'byte_width' cannot be NA")
+  expect_error(binary(-1), "'byte_width' must be > 0")
+  expect_error(binary("four"), class = "Rcpp::not_compatible")
+  expect_error(binary(c(2, 4)), class = "Rcpp::not_compatible")
 })
