@@ -20,47 +20,47 @@ context("File system")
 test_that("LocalFilesystem", {
   fs <- LocalFileSystem$create()
   DESCRIPTION <- system.file("DESCRIPTION", package = "arrow")
-  stat <- fs$GetTargetStats(DESCRIPTION)[[1]]
-  expect_equal(stat$base_name(), "DESCRIPTION")
-  expect_equal(stat$extension(), "")
-  expect_equal(stat$type, FileType$File)
-  expect_equal(stat$path, DESCRIPTION)
+  info <- fs$GetFileInfo(DESCRIPTION)[[1]]
+  expect_equal(info$base_name(), "DESCRIPTION")
+  expect_equal(info$extension(), "")
+  expect_equal(info$type, FileType$File)
+  expect_equal(info$path, DESCRIPTION)
   info <- file.info(DESCRIPTION)
 
-  expect_equal(stat$size, info$size)
+  expect_equal(info$size, info$size)
   # This fails due to a subsecond difference on Appveyor on Windows with R 3.3 only
   # So add a greater tolerance to allow for that
-  expect_equal(stat$mtime, info$mtime, tolerance = 1)
+  expect_equal(info$mtime, info$mtime, tolerance = 1)
 
   tf <- tempfile(fileext = ".txt")
   fs$CopyFile(DESCRIPTION, tf)
-  stat <- fs$GetTargetStats(tf)[[1]]
-  expect_equal(stat$extension(), "txt")
-  expect_equal(stat$size, info$size)
+  info <- fs$GetFileInfo(tf)[[1]]
+  expect_equal(info$extension(), "txt")
+  expect_equal(info$size, info$size)
   expect_equal(readLines(DESCRIPTION), readLines(tf))
 
   tf2 <- tempfile(fileext = ".txt")
   fs$Move(tf, tf2)
-  stats <- fs$GetTargetStats(c(tf, tf2, dirname(tf)))
-  expect_equal(stats[[1]]$type, FileType$NonExistent)
-  expect_equal(stats[[2]]$type, FileType$File)
-  expect_equal(stats[[3]]$type, FileType$Directory)
+  infos <- fs$GetFileInfo(c(tf, tf2, dirname(tf)))
+  expect_equal(infos[[1]]$type, FileType$NotFound)
+  expect_equal(infos[[2]]$type, FileType$File)
+  expect_equal(infos[[3]]$type, FileType$Directory)
 
   fs$DeleteFile(tf2)
-  expect_equal(fs$GetTargetStats(tf2)[[1L]]$type, FileType$NonExistent)
+  expect_equal(fs$GetFileInfo(tf2)[[1L]]$type, FileType$NotFound)
   expect_true(!file.exists(tf2))
 
-  expect_equal(fs$GetTargetStats(tf)[[1L]]$type, FileType$NonExistent)
+  expect_equal(fs$GetFileInfo(tf)[[1L]]$type, FileType$NotFound)
   expect_true(!file.exists(tf))
 
   td <- tempfile()
   fs$CreateDir(td)
-  expect_equal(fs$GetTargetStats(td)[[1L]]$type, FileType$Directory)
+  expect_equal(fs$GetFileInfo(td)[[1L]]$type, FileType$Directory)
   fs$CopyFile(DESCRIPTION, file.path(td, "DESCRIPTION"))
   fs$DeleteDirContents(td)
   expect_equal(length(dir(td)), 0L)
   fs$DeleteDir(td)
-  expect_equal(fs$GetTargetStats(td)[[1L]]$type, FileType$NonExistent)
+  expect_equal(fs$GetFileInfo(td)[[1L]]$type, FileType$NotFound)
 
   tf3 <- tempfile()
   os <- fs$OpenOutputStream(path = tf3)
@@ -85,19 +85,19 @@ test_that("SubTreeFilesystem", {
   expect_is(st_fs, "FileSystem")
   st_fs$CreateDir("test")
   st_fs$CopyFile("DESCRIPTION", "DESC.txt")
-  stats <- st_fs$GetTargetStats(c("DESCRIPTION", "test", "nope", "DESC.txt"))
-  expect_equal(stats[[1L]]$type, FileType$File)
-  expect_equal(stats[[2L]]$type, FileType$Directory)
-  expect_equal(stats[[3L]]$type, FileType$NonExistent)
-  expect_equal(stats[[4L]]$type, FileType$File)
-  expect_equal(stats[[4L]]$extension(), "txt")
+  infos <- st_fs$GetFileInfo(c("DESCRIPTION", "test", "nope", "DESC.txt"))
+  expect_equal(infos[[1L]]$type, FileType$File)
+  expect_equal(infos[[2L]]$type, FileType$Directory)
+  expect_equal(infos[[3L]]$type, FileType$NotFound)
+  expect_equal(infos[[4L]]$type, FileType$File)
+  expect_equal(infos[[4L]]$extension(), "txt")
 
   local_fs$DeleteDirContents(td)
-  stats <- st_fs$GetTargetStats(c("DESCRIPTION", "test", "nope", "DESC.txt"))
-  expect_equal(stats[[1L]]$type, FileType$NonExistent)
-  expect_equal(stats[[2L]]$type, FileType$NonExistent)
-  expect_equal(stats[[3L]]$type, FileType$NonExistent)
-  expect_equal(stats[[4L]]$type, FileType$NonExistent)
+  infos <- st_fs$GetFileInfo(c("DESCRIPTION", "test", "nope", "DESC.txt"))
+  expect_equal(infos[[1L]]$type, FileType$NotFound)
+  expect_equal(infos[[2L]]$type, FileType$NotFound)
+  expect_equal(infos[[3L]]$type, FileType$NotFound)
+  expect_equal(infos[[4L]]$type, FileType$NotFound)
 })
 
 test_that("LocalFileSystem + Selector", {
@@ -109,16 +109,16 @@ test_that("LocalFileSystem + Selector", {
   writeLines("...", file.path(td, "dir", "three.txt"))
 
   selector <- FileSelector$create(td, recursive = TRUE)
-  stats <- fs$GetTargetStats(selector)
-  expect_equal(length(stats), 4L)
-  types <- sapply(stats, function(.x) .x$type)
+  infos <- fs$GetFileInfo(selector)
+  expect_equal(length(infos), 4L)
+  types <- sapply(infos, function(.x) .x$type)
   expect_equal(sum(types == FileType$File), 3L)
   expect_equal(sum(types == FileType$Directory), 1L)
 
   selector <- FileSelector$create(td, recursive = FALSE)
-  stats <- fs$GetTargetStats(selector)
-  expect_equal(length(stats), 3L)
-  types <- sapply(stats, function(.x) .x$type)
+  infos <- fs$GetFileInfo(selector)
+  expect_equal(length(infos), 3L)
+  types <- sapply(infos, function(.x) .x$type)
   expect_equal(sum(types == FileType$File), 2L)
   expect_equal(sum(types == FileType$Directory), 1L)
 })
