@@ -30,11 +30,13 @@ test_that("reading a known Parquet file to tibble", {
 test_that("simple int column roundtrip", {
   df <- tibble::tibble(x = 1:5)
   pq_tmp_file <- tempfile() # You can specify the .parquet here but that's probably not necessary
-  on.exit(unlink(pq_tmp_file))
 
   write_parquet(df, pq_tmp_file)
   df_read <- read_parquet(pq_tmp_file)
-  expect_identical(df, df_read)
+  expect_equivalent(df, df_read)
+  # Make sure file connection is cleaned up
+  expect_error(file.remove(pq_tmp_file), NA)
+  expect_false(file.exists(pq_tmp_file))
 })
 
 test_that("read_parquet() supports col_select", {
@@ -118,7 +120,7 @@ test_that("Factors are preserved when writing/reading from Parquet", {
 
   write_parquet(df, pq_tmp_file)
   df_read <- read_parquet(pq_tmp_file)
-  expect_identical(df, df_read)
+  expect_equivalent(df, df_read)
 })
 
 test_that("Lists are preserved when writing/reading from Parquet", {
@@ -151,5 +153,5 @@ test_that("write_parquet() returns its input", {
   tf <- tempfile()
   on.exit(unlink(tf))
   df_out <- write_parquet(df, tf)
-  expect_identical(df, df_out)
+  expect_equivalent(df, df_out)
 })
