@@ -62,7 +62,7 @@ test_that("reading schema from Buffer", {
   expect_is(writer, "RecordBatchStreamWriter")
   writer$close()
 
-  buffer <- stream$getvalue()
+  buffer <- stream$finish()
   expect_is(buffer, "Buffer")
 
   reader <- MessageReader$create(buffer)
@@ -85,3 +85,27 @@ test_that("Input validation when creating a table with a schema", {
     "schema must be an arrow::Schema or NULL"
   )
 })
+
+test_that("Schema$Equals", {
+  a <- schema(b = double(), c = bool())
+  b <- a$WithMetadata(list(some="metadata"))
+
+  # different metadata
+  expect_failure(expect_equal(a, b))
+  expect_false(a$Equals(b, check_metadata = TRUE))
+
+  # Metadata not checked
+  expect_equivalent(a, b)
+
+  # Non-schema object
+  expect_false(a$Equals(42))
+})
+
+test_that("unify_schemas", {
+  a <- schema(b = double(), c = bool())
+  z <- schema(b = double(), k = utf8())
+  expect_equal(
+    unify_schemas(a, z),
+    schema(b = double(), c = bool(), k = utf8())
+  )
+})
