@@ -19,33 +19,33 @@ context("test-type")
 
 test_that("type() gets the right type for arrow::Array", {
   a <- Array$create(1:10)
-  expect_equal(type(a), a$type)
+  expect_type_equal(type(a), a$type)
 })
 
 test_that("type() gets the right type for ChunkedArray", {
   a <- chunked_array(1:10, 1:10)
-  expect_equal(type(a), a$type)
+  expect_type_equal(type(a), a$type)
 })
 
 test_that("type() infers from R type", {
-  expect_equal(type(1:10), int32())
-  expect_equal(type(1), float64())
-  expect_equal(type(TRUE), boolean())
-  expect_equal(type(raw()), int8())
-  expect_equal(type(""), utf8())
-  expect_equal(
+  expect_type_equal(type(1:10), int32())
+  expect_type_equal(type(1), float64())
+  expect_type_equal(type(TRUE), boolean())
+  expect_type_equal(type(raw()), int8())
+  expect_type_equal(type(""), utf8())
+  expect_type_equal(
     type(iris$Species),
     dictionary(int8(), utf8(), FALSE)
   )
-  expect_equal(
+  expect_type_equal(
     type(lubridate::ymd_hms("2019-02-14 13:55:05")),
-    timestamp(TimeUnit$MICRO, "GMT")
+    timestamp(TimeUnit$MICRO, "UTC")
   )
-  expect_equal(
+  expect_type_equal(
     type(hms::hms(56, 34, 12)),
     time32(unit = TimeUnit$SECOND)
   )
-  expect_equal(
+  expect_type_equal(
     type(bit64::integer64()),
     int64()
   )
@@ -53,5 +53,18 @@ test_that("type() infers from R type", {
 
 test_that("type() can infer struct types from data frames", {
   df <- tibble::tibble(x = 1:10, y = rnorm(10), z = letters[1:10])
-  expect_equal(type(df), struct(x = int32(), y = float64(), z = utf8()))
+  expect_type_equal(type(df), struct(x = int32(), y = float64(), z = utf8()))
+})
+
+test_that("DataType$Equals", {
+  a <- int32()
+  b <- int32()
+  z <- float64()
+  expect_true(a == b)
+  expect_true(a$Equals(b))
+  expect_false(a == z)
+  expect_equal(a, b)
+  expect_failure(expect_equal(a, z))
+  expect_failure(expect_type_equal(a, z), "int32 not equal to double")
+  expect_false(a$Equals(32L))
 })
